/* 
 * ledcpumeter.c
 * Reads cpu states from /proc/stat
 * (C) 1997 by Matthias Cramer (cramer@freestone.ch)
 * Distributed under GPL
 */

#include <stdio.h>
#include <sys/time.h>
#include <fcntl.h>
#include "config.h"
#include "port.h"


/* File handlers */
static int pstat;

/* Prototypes */

void getload(void);
void my_sleep(float secs); 
void put_leds(int percentage,int hi_percentage);


void main()
{
  static int state;

  if(ioperm(OUTPORT,1,1)) {
    fprintf(stderr,"ioperm error for OUTPORT.\n");
    exit(1);
  }
          
  pstat = open("/proc/stat", O_RDONLY);
  if(pstat == -1) perror("open /proc/stat"), exit(1);

  while(1) {
    getload();
    my_sleep(UPDATE_DELAY); 
  }
}


void getload(void)
{
   int len, status;
   int user, nice, system, idle;
   static int prev_user, prev_nice, prev_system, prev_idle;
   int delta_user, delta_nice, delta_system, delta_idle;
   int full,load;
   float percentage, avg_percentage;
   static float hi_percentage;
   static int ring_size=RING_SIZE, hi_delay=PEAK_DELAY;
   static float ring_buffer[RING_SIZE];
   static int hi_count, ring_counter;
   char buf[1000], dummy[50];

   lseek(pstat, 0, SEEK_SET);
   len = read(pstat, buf, sizeof(buf));
   buf[len] = 0;
   status = sscanf(buf, "%s %d %d %d %d", dummy, &user, &nice, &system, &idle);
   
   if(status == EOF)
      perror("sscanf /proc/stat"), exit(1);
   if(status != 5)
      fprintf(stderr, "sscanf /proc/stat returned %d\n", status), exit(1);

#ifdef DEBUG
   printf("CPU:  idle=%d user=%d nice=%d system=%d\n", idle, user, nice, system);
#endif

   delta_idle=idle-prev_idle;
   delta_user=user-prev_user;
   delta_nice=nice-prev_nice;
   delta_system=system-prev_system;

#ifdef DEBUG
   printf("CPU1: idle=%d user=%d nice=%d system=%d\n",delta_idle,delta_user,delta_nice,delta_system);
#endif

   full=delta_idle+delta_user+delta_nice+delta_system;
   load=full-delta_idle;
   percentage=(load*1.0/full*1.0)*100;
   --hi_count;
   if ((hi_percentage<=percentage) || (hi_count<=0))
      {
        hi_percentage=percentage;
        hi_count=hi_delay;
      }
   ring_buffer[ring_counter]=percentage;
   ++ring_counter;
   if (ring_counter>=ring_size) ring_counter=0;

   avg_percentage=0.0;   
   for (len=0; len<ring_size; len++)
       avg_percentage+=ring_buffer[len];
   avg_percentage/=ring_size;

#ifdef DEBUG
   printf("hi_count:%d, hi_percentage:%g\n",hi_count,hi_percentage);
   printf("FullScale:%d Load=%d PercetageLoad:%g avg_percentage:%g\n",full,load,percentage, avg_percentage);
#endif
   
   if (avg_percentage-5<percentage)
      put_leds((int)percentage, (int)hi_percentage);
    else
      put_leds((int)avg_percentage, (int)hi_percentage);

   prev_user=user;
   prev_nice=nice;
   prev_system=system;
   prev_idle=idle;

}

void put_leds(int percentage, int hi_percentage)
{
  int numoflights,map,i,highlight;
  
  numoflights=(int)(percentage*1.0/(100/8));
  highlight=(int)(hi_percentage*1.0/(100/8));

#ifdef DEBUG
  printf("lights=%d, highlight=%d\n",numoflights, highlight);
#endif

  map=0;
  for(i=0;i<numoflights;i++)
     map |= (1<<i);
     
     map |= (1<<highlight-1);
            
  port_out(OUTPORT,map);

}


void my_sleep(float secs)
{

 struct timeval tv;
 int retval;

 tv.tv_sec = (int)secs;
 tv.tv_usec = (int)((secs - tv.tv_sec*1.0) * 1000000);

 retval=select(0, 0, 0, 0, &tv);

}
